/*
 * FunctionDialog.java - Dialog class of the jQuantum computer simulator
 *
 * Copyright (C) 2004-2008 Andreas de Vries
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses
 * or write to the Free Software Foundation,Inc., 51 Franklin Street,
 * Fifth Floor, Boston, MA 02110-1301  USA
 */
package jquantum;
import java.util.Properties;
import javax.swing.*;
import org.mathIT.util.FunctionParser;
/**
 * This class enables to input of an arbitrary function for function evaluation to be
 * added to a quantum circuit.
 * @author  Andreas de Vries
 * @version 1.3
 */
public class FunctionDialog extends javax.swing.JDialog {
	private static final long serialVersionUID = 1748297568;
    private Properties bundle;

    /** The specified function.*/
    public FunctionParser function;
    /** Flag indicating whether the cancel button has been pressed.*/
    public boolean cancelButtonClicked = false;
    
    /** Creates new form InitDialog */
    public FunctionDialog(java.awt.Frame parent, boolean modal, Properties bundle) {
        super(parent, modal);
        this.bundle = bundle;
        
        try {
           UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
           if ( UIManager.getSystemLookAndFeelClassName().equals( "javax.swing.plaf.metal.MetalLookAndFeel" ) ) {
              UIManager.setLookAndFeel( "com.sun.java.swing.plaf.motif.MotifLookAndFeel" );
           }
        } catch ( Exception e ) {
           //System.err.println( "Look & Feel Error: " + e.toString() );
        }
        
        initComponents();

        questionLabel.setText(bundle.getProperty("FunctionDialog.questionLabel.text"));
        okButton.setText(bundle.getProperty("okButton.text"));
        cancelButton.setText(bundle.getProperty("cancelButton.text"));
        
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width  - getSize().width )/2,
                    (screenSize.height - getSize().height)/2);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        headPanel = new javax.swing.JPanel();
        questionLabel = new javax.swing.JLabel();
        inputPanel = new javax.swing.JPanel();
        functionLabel = new javax.swing.JLabel();
        functionTextField = new javax.swing.JTextField();
        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();

        addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
			public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        questionLabel.setText("Input the function to be evaluated:");
        headPanel.add(questionLabel);

        getContentPane().add(headPanel, java.awt.BorderLayout.NORTH);

        functionLabel.setText("<html><i>f</i>(<i>x</i>) = ");
        inputPanel.add(functionLabel);

        functionTextField.setColumns(10);
        functionTextField.setText("z^x mod 15");
        functionTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                functionTextFieldActionPerformed(evt);
            }
        });

        inputPanel.add(functionTextField);

        getContentPane().add(inputPanel, java.awt.BorderLayout.CENTER);

        okButton.setText(" OK ");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(okButton);

        cancelButton.setText(" Cancel ");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(cancelButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

        pack();
    }//GEN-END:initComponents

    private void functionTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_functionTextFieldActionPerformed
        setValues();
    }//GEN-LAST:event_functionTextFieldActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        cancelButtonClicked = true;
        setVisible(false);
        removeAll();
        dispose();
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        setValues();
    }//GEN-LAST:event_okButtonActionPerformed

    /**
     * Returns the specified function.
     * @return the specified function
     */
    public FunctionParser getFunction() {
        return function;
    }

    private void setValues() {
       cancelButtonClicked = false;
       try {
          String input = functionTextField.getText();
          if ( !FunctionParser.checkSyntax( input ) )
              throw new IllegalArgumentException("Function syntax wrong: "+input);
          function = new FunctionParser( input );
          setVisible(false);
       } catch ( Exception e ) {
           String title = bundle.getProperty("errorMessage.title.text");
           String output = "<html><h3 style=\"color:red\">";
           output += bundle.getProperty("FunctionDialog.errorMessage") + "!";
           output += "</h3><table border=\"0\" style=\"text-align:center\" valign=\"top\">";
           output += "<tr><td>";
           output += "<table border=\"1\" style=\"text-align:center\">";
           output += "<tr><th colspan=\"2\" style=\"text-align:left\">";
           output += bundle.getProperty("constants.text");
           output += "</th></tr>";
           for ( int i = 0; i < FunctionParser.constantName.length; i++ ) {
              output += "<tr><td>";
              output += FunctionParser.constantName[i];
              output += "</td><td>";
              output += FunctionParser.constantValue[i];
              output += "</td></tr>";
           }
           output += "</table>";
           output += "</td><td>";
           output += "<table border=\"1\" style=\"text-align:center\">";
           output += "<tr><th colspan=\"5\" style=\"text-align:left\">";
           output += bundle.getProperty("functions.text");
           output += "</th></tr>";
           output += showOperators(1);
           output += "</td><td>";
           output += "<table border=\"1\" style=\"text-align:center\">";
           output += "<tr><th colspan=\"5\" style=\"text-align:left\">";
           output += bundle.getProperty("binaryOperators.text");
           output += "</th></tr>";
           output += showOperators(2);
           output += "</td><td>";
           output += "<table border=\"1\" style=\"text-align:center\">";
           output += "<tr><th colspan=\"5\" style=\"text-align:left\">";
           output += bundle.getProperty("ternaryOperators.text");
           output += "</th></tr>";
           output += showOperators(3);
           output += "</td></tr></table>";
           output += "</html>";

           JOptionPane.showMessageDialog( 
             this, output, title, JOptionPane.PLAIN_MESSAGE
           );
       }
    }

    private String showOperators( int n ) {
       if ( FunctionParser.operator.length <= n ) 
          return "";
       String output = "";
       for ( int i = 0; i < FunctionParser.operator[n].length; i++ ) {
          if ( i % 5 == 0 ) {
             output += "<tr>";
          }
          output += "<td>";
          output += ( FunctionParser.operator[n][i] ).replaceAll("<","&lt;");
          output += "</td>";
          if ( i % 5 == 4 ) {
             output += "</tr>";
          }
       }
       output += "</table>";
       return output;
    }

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        cancelButtonClicked = true;
        setVisible(false);
        removeAll();
        dispose();
    }//GEN-LAST:event_closeDialog
    
    /*
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
               String language = "fr"; //java.util.Locale.getDefault().getLanguage();
               java.util.Properties bundle = new java.util.Properties();
               try {
                  System.out.print("Try to load " + "/jquantum/Bundle_" + language + ".xml ...");
                  bundle.loadFromXML(getClass().getResourceAsStream("/jquantum/Bundle_" + language + ".xml"));
                  System.out.println(" successfully!");
               } catch(Exception e) {
                  try {
                     bundle.loadFromXML(getClass().getResourceAsStream("/jquantum/Bundle_en.xml"));
                     System.out.println(" but loaded /jquantum/Bundle_en.xml!");
                  } catch (Exception e2) {
                     e2.printStackTrace();
                  }
               }
               new FunctionDialog(new javax.swing.JFrame(), true, bundle).setVisible(true);
               //System.exit(0);
            }
        });
    }
    // */
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel functionLabel;
    private javax.swing.JTextField functionTextField;
    private javax.swing.JPanel headPanel;
    private javax.swing.JPanel inputPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel questionLabel;
    // End of variables declaration//GEN-END:variables
}
