/*
 * InitDialog.java - Dialog class of the jQuantum computer simulator
 *
 * Copyright (C) 2004-2008 Andreas de Vries
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses
 * or write to the Free Software Foundation,Inc., 51 Franklin Street,
 * Fifth Floor, Boston, MA 02110-1301  USA
 */
package jquantum;
import java.util.Properties;
import javax.swing.*;
/**
 * This class enables a dialog frame to create a quantum circuit.
 * @author  Andreas de Vries
 * @version 1.3
 */
public class InitDialog extends javax.swing.JDialog {
    private static final long serialVersionUID = 1812340376;
    Properties bundle;
    /** The size of the <i>x</i>-register.*/
    public int xRegisterSize;
    /** The size of the <i>y</i>-register.*/
    public int yRegisterSize;
    /** Represents the initial state of the circuit.*/
    public int initialState = 0;
    /** Flag indicating whether the cancel button has been pressed.*/
    public boolean cancelButtonClicked = false;
    
    /** Creates new form InitDialog */
    public InitDialog(java.awt.Frame parent, boolean modal, Properties bundle) {
        super(parent, modal);
        this.bundle = bundle;
        try {
           UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
           if ( UIManager.getSystemLookAndFeelClassName().equals( "javax.swing.plaf.metal.MetalLookAndFeel" ) ) {
              UIManager.setLookAndFeel( "com.sun.java.swing.plaf.motif.MotifLookAndFeel" );
           }
        } catch ( Exception e ) {
           //System.err.println( "Look & Feel Error: " + e.toString() );
        }
        
        initComponents();

        questionLabel.setText(bundle.getProperty("InitDialog.questionLabel.text"));
        iniLabel.setText(bundle.getProperty("InitDialog.iniLabel.text"));
        okButton.setText(bundle.getProperty("okButton.text"));
        cancelButton.setText(bundle.getProperty("cancelButton.text"));
        xLabel.setText("<html><i>x</i>-" + bundle.getProperty("Register.text") + ": </html>");
        yLabel.setText("<html><i>y</i>-" + bundle.getProperty("Register.text") + ": </html>");
        
        pack();
        
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width  - getSize().width )/2,
                    (screenSize.height - getSize().height)/2);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        headPanel = new javax.swing.JPanel();
        questionLabel = new javax.swing.JLabel();
        inputPanel = new javax.swing.JPanel();
        xPanel = new javax.swing.JPanel();
        xLabel = new javax.swing.JLabel();
        xTextField = new javax.swing.JTextField();
        fillLabel1 = new javax.swing.JLabel();
        yPanel = new javax.swing.JPanel();
        yLabel = new javax.swing.JLabel();
        yTextField = new javax.swing.JTextField();
        fillLabel2 = new javax.swing.JLabel();
        initialStatePanel = new javax.swing.JPanel();
        iniLabel = new javax.swing.JLabel();
        iniTextField = new javax.swing.JTextField();
        fillLabel3 = new javax.swing.JLabel();
        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();

        addWindowListener(new java.awt.event.WindowAdapter() {
            @Override
			public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        headPanel.setLayout(new java.awt.BorderLayout());

        headPanel.setBorder(new javax.swing.border.EtchedBorder());
        questionLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        questionLabel.setText("Please input the register sizes:");
        headPanel.add(questionLabel, java.awt.BorderLayout.NORTH);

        inputPanel.setLayout(new java.awt.GridLayout(2, 1));

        xPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        xLabel.setText("<html><i>x</i>-register: </html>");
        xPanel.add(xLabel);

        xTextField.setColumns(3);
        xTextField.setText("8");
        xTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                xTextFieldActionPerformed(evt);
            }
        });

        xPanel.add(xTextField);

        fillLabel1.setText("          ");
        xPanel.add(fillLabel1);

        inputPanel.add(xPanel);

        yPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        yLabel.setText("<html><i>y</i>-register: </html>");
        yPanel.add(yLabel);

        yTextField.setColumns(3);
        yTextField.setText("5");
        yTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                yTextFieldActionPerformed(evt);
            }
        });

        yPanel.add(yTextField);

        fillLabel2.setText("          ");
        yPanel.add(fillLabel2);

        inputPanel.add(yPanel);

        headPanel.add(inputPanel, java.awt.BorderLayout.CENTER);

        getContentPane().add(headPanel, java.awt.BorderLayout.NORTH);

        initialStatePanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        initialStatePanel.setBorder(new javax.swing.border.EtchedBorder());
        iniLabel.setText("Initial state: ");
        initialStatePanel.add(iniLabel);

        iniTextField.setColumns(3);
        iniTextField.setText("0");
        iniTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                iniTextFieldActionPerformed(evt);
            }
        });

        initialStatePanel.add(iniTextField);

        fillLabel3.setText("          ");
        initialStatePanel.add(fillLabel3);

        getContentPane().add(initialStatePanel, java.awt.BorderLayout.CENTER);

        okButton.setText(" OK ");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(okButton);

        cancelButton.setText(" Cancel ");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(cancelButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);

        pack();
    }//GEN-END:initComponents

    private void iniTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_iniTextFieldActionPerformed
        setValues();
    }//GEN-LAST:event_iniTextFieldActionPerformed

    private void yTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_yTextFieldActionPerformed
        setValues();
    }//GEN-LAST:event_yTextFieldActionPerformed

    private void xTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_xTextFieldActionPerformed
        setValues();
    }//GEN-LAST:event_xTextFieldActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        cancelButtonClicked = true;
        setVisible(false);
        removeAll();
        dispose();
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        setValues();
    }//GEN-LAST:event_okButtonActionPerformed

    /**
     * Returns an array consisting of the three entries 
     * {@link #xRegisterSize}, {@link #yRegisterSize}, {@link #initialState}.
     * @return an array consisting of the three entries 
     * {@link #xRegisterSize}, {@link #yRegisterSize}, {@link #initialState}
     */
    public int[] getValues() {
        return new int[] { xRegisterSize, yRegisterSize, initialState };
    }

    private void setValues() {
        cancelButtonClicked = false;
        try {
            xRegisterSize = Integer.parseInt( xTextField.getText() );
            if (xRegisterSize <= 0) throw new Exception();
            try {
                yRegisterSize = Integer.parseInt( yTextField.getText() );
                if (yRegisterSize < 0) throw new Exception();
                try {
                    initialState = Integer.parseInt( iniTextField.getText() );
                    if ( initialState > ( 1 << xRegisterSize ) ) throw new Exception();
                    setVisible(false);
                } catch ( Exception ei ) {
                    String message = bundle.getProperty("InitDialog.errorMessage_1") +
                    " = " + (1 << xRegisterSize) + "!";
                    JOptionPane.showMessageDialog(null, message);
                }
            } catch ( Exception ey ) {
                String message = bundle.getProperty("InitDialog.errorMessage_2");
                JOptionPane.showMessageDialog(null, message);
            }
        } catch ( Exception ex ) {
            String message = bundle.getProperty("InitDialog.errorMessage_3");
            JOptionPane.showMessageDialog(null, message);
        }
    }
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        cancelButtonClicked = true;
        setVisible(false);
        removeAll();
        dispose();
    }//GEN-LAST:event_closeDialog
    
    /*
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
               String language = "fr"; //java.util.Locale.getDefault().getLanguage();
               java.util.Properties bundle = new java.util.Properties();
               try {
                  System.out.print("Try to load " + "/jquantum/resources/Bundle_" + language + ".xml ...");
                  bundle.loadFromXML(getClass().getResourceAsStream("/jquantum/resources/Bundle_" + language + ".xml"));
                  System.out.println(" successfully!");
               } catch(Exception e) {
                  try {
                     bundle.loadFromXML(getClass().getResourceAsStream("/jquantum/resources/Bundle_en.xml"));
                     System.out.println(" but loaded /jquantum/resources/Bundle_en.xml!");
                  } catch (Exception e2) {
                     e2.printStackTrace();
                  }
               }
               new InitDialog(new javax.swing.JFrame(), true, bundle).setVisible(true);
               System.exit(0);
            }
        });
    }
    // */
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel fillLabel1;
    private javax.swing.JLabel fillLabel2;
    private javax.swing.JLabel fillLabel3;
    private javax.swing.JPanel headPanel;
    private javax.swing.JLabel iniLabel;
    private javax.swing.JTextField iniTextField;
    private javax.swing.JPanel initialStatePanel;
    private javax.swing.JPanel inputPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel questionLabel;
    private javax.swing.JLabel xLabel;
    private javax.swing.JPanel xPanel;
    private javax.swing.JTextField xTextField;
    private javax.swing.JLabel yLabel;
    private javax.swing.JPanel yPanel;
    private javax.swing.JTextField yTextField;
    // End of variables declaration//GEN-END:variables
    
}
